@@ -1,16 +1,24 @@
 #' Calculate distance between sample data before and after registration
 #'
-#' @param results Result of registration process using \code{\link{register}}.
+#' \code{calculate_distance()} is a function that allows users to calculate
+#' pairwise distances between samples from different time points to
+#' investigate the similarity of progression before or after registration.
 #'
-#' @return This function returns a list of data frames which includes:
+#' @param results Result of registration process using [register()].
+#' @param type Whether to calculate distance considering only "registered" genes (default) or "all" genes.
+#' @param genes_list Optional vector indicating the \code{gene_id} values to be considered.
 #'
-#' \item{registered}{distance between scaled reference and query expressions using registered time points.}
-#' \item{original}{distance between scaled reference and query expressions using original time points.}
+#' @return This function returns a \code{dist_greatR} object containing two data frames:
+#'
+#' \item{registered}{pairwise distance between scaled reference and query expressions using registered time points.}
+#' \item{original}{pairwise distance between scaled reference and query expressions using original time points.}
 #'
 #' @export
-calculate_distance <- function(results) {
+calculate_distance <- function(results, type = c("registered", "all"), genes_list = NULL) {
   # Suppress "no visible binding for global variable" note
   gene_id <- NULL
+  gene_ref <- NULL
+  gene_query <- NULL
   accession <- NULL
   timepoint <- NULL
   timepoint_reg <- NULL
@@ -20,36 +28,67 @@ calculate_distance <- function(results) {
   exp_ref <- NULL
   exp_query <- NULL
 
+  # Validate parameters
+  type <- match.arg(type)
+
   # Retrieve data from results
   data <- results$data
   reference <- attr(data, "ref")
   query <- attr(data, "query")
-  data <- unique(data[, .(expression_value = mean(expression_value)), by = .(gene_id, accession, timepoint, timepoint_reg)])
 
-  data_query <- data[data$accession == query]
-  data_ref <- data[data$accession == reference]
+  # Parse data
+  if (type == "registered") {
+    reg_genes <- results$model_comparison[results$model_comparison$registered, gene_id]
+    data <- data[data$gene_id %in% reg_genes, ]
+  }
+
+  # Select genes to be considered
+  if (any(!is.null(genes_list))) {
+    if (!inherits(genes_list, "character")) {
+      stop(
+        cli::format_error(c(
+          "{.var genes_list} must be a {.cls character} vector.",
+          "x" = "You supplied vectors with {.cls {class(genes_list)}} values."
+        )),
+        call. = FALSE
+      )
+    }
+
+    data <- data[data$gene_id %in% genes_list]
+  }
+
+  # Split data into referene and query
+  data <- unique(data[, .(expression_value = mean(expression_value)), by = .(gene_id, accession, timepoint, timepoint_reg)])
+  data_query <- data[data$accession == query][, .(gene_query = gene_id, accession, timepoint_query = timepoint, timepoint_reg, exp_query = expression_value)]
+  data_ref <- data[data$accession == reference][, .(gene_ref = gene_id, accession, timepoint_ref = timepoint, timepoint_reg, exp_ref = expression_value)]
 
   # Cross join all reference and query time points
-  timepoint_cj_registered <- get_timepoint_comb_registered_data(data_ref, data_query)
-  timepoint_cj_original <- get_timepoint_comb_original_data(data_ref, data_query)
+  timepoint_cj_result <- get_timepoint_comb_result_data(
+    data_ref[, .(gene_ref, timepoint_ref, exp_ref)],
+    data_query[, .(gene_query, timepoint_query, timepoint_reg, exp_query)]
+  )
+  timepoint_cj_original <- get_timepoint_comb_original_data(
+    data_ref[, .(gene_ref, timepoint_ref, exp_ref)],
+    data_query[, .(gene_query, timepoint_query, exp_query)]
+  )
 
   # Calculate mean square distances
-  dist_registered <- timepoint_cj_registered[, .(distance = mean((exp_ref - exp_query)^2)), by = .(timepoint_ref, timepoint_query)][timepoint_query >= 0]
+  dist_result <- timepoint_cj_result[, .(distance = mean((exp_ref - exp_query)^2)), by = .(timepoint_ref, timepoint_query)][timepoint_query >= 0]
   dist_original <- timepoint_cj_original[, .(distance = mean((exp_ref - exp_query)^2)), by = .(timepoint_ref, timepoint_query)][timepoint_query >= 0]
 
   # Add accession values as data attributes
-  data.table::setattr(dist_registered, "ref", reference)
-  data.table::setattr(dist_registered, "query", query)
+  data.table::setattr(dist_result, "ref", reference)
+  data.table::setattr(dist_result, "query", query)
   data.table::setattr(dist_original, "ref", reference)
   data.table::setattr(dist_original, "query", query)
 
   # Results object
   results_list <- list(
-    registered = dist_registered,
+    result = dist_result,
     original = dist_original
   )
 
-  return(results_list)
+  return(new_dist_greatR(results_list))
 }
 
 #' Cross join all original reference and query time points and expression values
@@ -68,17 +107,15 @@ get_timepoint_comb_original_data <- function(data_ref, data_query) {
   exp_query <- NULL
 
   # Perform cross join
-  genes <- unique(data_query$gene_id)
+  genes <- unique(data_query$gene_query)
 
   comb <- lapply(
     genes,
     function(gene) {
-      comb_gene <- cross_join(
-        unique(data_ref[data_ref$gene_id == gene][, .(gene_ref = gene_id, timepoint_ref = timepoint, exp_ref = expression_value)]),
-        unique(data_query[data_query$gene_id == gene][, .(gene_query = gene_id, timepoint_query = timepoint, exp_query = expression_value)])
+      cross_join(
+        unique(data_ref[data_ref$gene_ref == gene]),
+        unique(data_query[data_query$gene_query == gene])
       )
-
-      return(comb_gene)
     }
   )
 
@@ -93,7 +130,7 @@ get_timepoint_comb_original_data <- function(data_ref, data_query) {
 #' Cross join all reference and registered query time points and expression values
 #'
 #' @noRd
-get_timepoint_comb_registered_data <- function(data_ref, data_query) {
+get_timepoint_comb_result_data <- function(data_ref, data_query) {
   # Suppress "no visible binding for global variable" note
   gene_id <- NULL
   gene_ref <- NULL
@@ -107,28 +144,26 @@ get_timepoint_comb_registered_data <- function(data_ref, data_query) {
   exp_query <- NULL
 
   # The imputed query time points to estimate expression values for
-  timepoint_ranges_query <- data_query[, .(min_t = ceiling(min(timepoint_reg)), max_t = floor(max(timepoint_reg))), by = "gene_id"]
+  timepoint_ranges_query <- data_query[, .(min_t = ceiling(min(timepoint_reg)), max_t = floor(max(timepoint_reg))), by = "gene_query"]
 
   imputed_query_timepoints <- data.table::rbindlist(
     Map(
       function(x, min_t, max_t) {
         data.table::data.table(gene_query = rep(x, max_t - min_t + 1), timepoint_query = seq(min_t, max_t))
-      }, timepoint_ranges_query$gene_id, timepoint_ranges_query$min_t, timepoint_ranges_query$max_t
+      }, timepoint_ranges_query$gene_query, timepoint_ranges_query$min_t, timepoint_ranges_query$max_t
     )
   )
 
   # Perform cross join
-  genes <- unique(data_query$gene_id)
+  genes <- unique(data_query$gene_query)
 
   comb <- lapply(
     genes,
     function(gene) {
-      comb_gene <- cross_join(
-        unique(data_ref[data_ref$gene_id == gene][, .(gene_ref = gene_id, timepoint_ref = timepoint, exp_ref = expression_value)]),
+      cross_join(
+        unique(data_ref[data_ref$gene_ref == gene]),
         imputed_query_timepoints[imputed_query_timepoints$gene_query == gene]
       )
-
-      return(comb_gene)
     }
   )
 
@@ -138,7 +173,7 @@ get_timepoint_comb_registered_data <- function(data_ref, data_query) {
   fits <- lapply(
     genes,
     function(gene) {
-      fit_spline_model(data_query[data_query$gene_id == gene], x = "timepoint_reg")
+      fit_spline_model(data_query[data_query$gene_query == gene], x = "timepoint_reg", y = "exp_query")
     }
   )
   names(fits) <- genes
@@ -160,3 +195,13 @@ get_timepoint_comb_registered_data <- function(data_ref, data_query) {
 
   return(comb)
 }
+
+new_dist_greatR <- function(x) {
+  structure(x, class = c("dist_greatR", class(x)))
+}
+
+#' @export
+print.dist_greatR <- function(x, ...) {
+  print(x[seq_along(x)])
+  return(invisible(x))
+}
