@@ -5,8 +5,9 @@
 #'
 #' @docType package
 #' @name greatR
+#' @aliases greatR-package
 #' @noRd
-NULL
+"_PACKAGE"
 
 ## quiets concerns of R CMD check re: the .'s that appear in pipelines
 if (getRversion() >= "2.15.1") {
