@@ -4,29 +4,30 @@
 #' wishes to compare.
 #'
 #' @param input Input data frame containing all replicates of gene expression in each genotype at each time point.
-#' @param stretches Candidate registration stretch factors to apply to query data, only required if \code{optimise_registration_parameters = FALSE}.
-#' @param shifts Candidate registration shift values to apply to query data, only required if \code{optimise_registration_parameters = FALSE}.
+#' @param stretches Candidate registration stretch factors to apply to query data, only required if \code{use_optimisation = FALSE}.
+#' @param shifts Candidate registration shift values to apply to query data, only required if \code{use_optimisation = FALSE}.
 #' @param reference Accession name of reference data.
 #' @param query Accession name of query data.
 #' @param scaling_method Scaling method applied to data prior to registration process. Either \code{none} (default), \code{z-score}, or \code{min-max}.
-#' @param overlapping_percent Minimum percentage of overlapping time points on the reference data. Shifts will be only considered if it leaves at least this percentage of overlapping time points after applying the registration function.
-#' @param optimise_registration_parameters Whether to optimise registration parameters. By default, \code{TRUE}.
-#' @param optimisation_method Optimisation method to use. Either \code{"nm"} for Nelder-Mead (default), \code{"lbfgsb"} for L-BFGS-B, or \code{"sa"} for Simulated Annealing.
+#' @param overlapping_percent Minimum percentage of overlapping time point range of the reference data. Shifts will be only considered if it leaves at least this percentage of overlapping time point range after applying the registration.
+#' @param use_optimisation Whether to optimise registration parameters. By default, \code{TRUE}.
+#' @param optimisation_method Optimisation method to use. Either \code{"lbfgsb"} for L-BFGS-B (default), \code{"nm"} for Nelder-Mead, or \code{"sa"} for Simulated Annealing.
 #' @param optimisation_config Optional list with arguments to override the default optimisation configuration.
 #' @param exp_sd Optional experimental standard deviation on the expression replicates.
 #' @param num_cores Number of cores to use if the user wants to register genes asynchronously (in parallel) in the background on the same machine. By default, \code{NA}, the registration will be run without parallelisation.
 #'
-#' @return This function returns a list of data frames, containing:
+#' @return This function returns a \code{res_greatR} object containing:
 #'
 #' \item{data}{a table containing the scaled input data and an additional \code{timepoint_reg} column after applying registration parameters to the query data.}
 #' \item{model_comparison}{a table comparing the optimal registration function for each gene (based on `all_shifts_df` scores) to model with no registration applied.}
+#' \item{fun_args}{a list of arguments used when calling the function.}
 #'
 #' @export
 #'
 #' @examples
 #' \dontrun{
 #' # Load a data frame from the sample data
-#' data_path <- system.file("extdata/brapa_arabidopsis_all_replicates.csv", package = "greatR")
+#' data_path <- system.file("extdata/brapa_arabidopsis_data.csv", package = "greatR")
 #' all_data <- utils::read.csv(data_path)
 #'
 #' # Running the registration
@@ -43,11 +44,17 @@ register <- function(input,
                      query,
                      scaling_method = c("none", "z-score", "min-max"),
                      overlapping_percent = 50,
-                     optimise_registration_parameters = TRUE,
-                     optimisation_method = c("nm", "lbfgsb", "sa"),
+                     use_optimisation = TRUE,
+                     optimisation_method = c("lbfgsb", "nm", "sa"),
                      optimisation_config = NULL,
                      exp_sd = NA,
                      num_cores = NA) {
+  # Store function arguments
+  fun_args <- c(as.list(environment()))
+  fun_args$input <- NULL
+  fun_args$scaling_method <- fun_args$scaling_method[1]
+  fun_args$optimisation_method <- fun_args$optimisation_method[1]
+
   # Suppress "no visible binding for global variable" note
   gene_id <- NULL
   accession <- NULL
@@ -95,45 +102,32 @@ register <- function(input,
     loglik_separate <- calc_loglik_H2(gene_data)
 
     # Explore search space
-    best_params <- explore_manual_search_space(gene_data, stretches, shifts, loglik_separate)
+    best_params <- explore_manual_search_space(gene_data, stretches, shifts, loglik_separate, overlapping_percent)
 
     # Register for Hypothesis 1
-    results <- register_manually(gene_data, best_params$stretch, best_params$shift, loglik_separate)
+    results <- register_manually(gene_data, best_params$stretch, best_params$shift, loglik_separate, overlapping_percent)
 
     return(results)
   }
 
-  # Validate input data
-  cli::cli_h1("Validating input data")
-
-  match_names(
-    x = colnames(input),
-    lookup = c("gene_id", "accession", "timepoint", "expression_value", "replicate"),
-    error = "Must review the column names of your input data:",
-    name_string = "column names"
-  )
-
-  match_names(
-    x = c(reference, query),
-    lookup = unique(input$accession),
-    error = "Must review the supplied {.var reference} and {.var query} values:",
-    name_string = "accession values"
-  )
-
   # Preprocess data
   overlapping_percent <- overlapping_percent / 100
+  input <- transform_input(input, reference, query)
   all_data <- preprocess_data(input, reference, query, exp_sd, scaling_method)
   gene_id_list <- unique(all_data$gene_id)
 
   # Begin registration logic
-  if (optimise_registration_parameters) {
+  if (use_optimisation) {
     optimisation_method <- match.arg(optimisation_method)
 
     # Registration with optimisation
     cli::cli_h1("Starting registration with optimisation")
 
     # Select optimisation method
-    if (optimisation_method == "nm") {
+    if (optimisation_method == "lbfgsb") {
+      cli::cli_alert_info("Using L-BFGS-B optimisation method.")
+      optimise_fun <- optimise_using_lbfgsb
+    } else if (optimisation_method == "nm") {
       cli::cli_alert_info("Using Nelder-Mead method.")
       optimise_fun <- optimise_using_nm
       if (is.null(optimisation_config)) {
@@ -145,15 +139,13 @@ register <- function(input,
       if (is.null(optimisation_config)) {
         optimisation_config <- list(num_iterations = 60, num_fun_evals = 100)
       }
-    } else if (optimisation_method == "lbfgsb") {
-      cli::cli_alert_info("Using L-BFGS-B optimization method.")
-      optimise_fun <- optimise_using_lbfgsb
     }
 
     # Validate stretch and shift values
     validate_params(stretches, shifts, "optimisation")
 
     # Run optimisation
+    cli::cli_alert_info("Using {.var overlapping_percent} = {overlapping_percent * 100}% as a registration criterion.")
     if (is.na(num_cores)) {
       results <- lapply(
         cli::cli_progress_along(
@@ -181,6 +173,7 @@ register <- function(input,
     validate_params(stretches, shifts, "manual")
 
     # Apply manual registration
+    cli::cli_alert_info("Using {.var overlapping_percent} = {overlapping_percent * 100}% as a registration criterion.")
     if (is.na(num_cores)) {
       results <- lapply(
         cli::cli_progress_along(
@@ -226,23 +219,21 @@ register <- function(input,
   all_data <- all_data[order(gene_id, accession, timepoint, replicate)]
 
   # Restore original query and reference accession names
-  all_data[, c("time_delta", "timepoint_id") := NULL]
+  all_data[, c("timepoint_id") := NULL]
   all_data[, accession := factor(accession, levels = c("ref", "query"), labels = c(reference, query))][, .(gene_id, accession, timepoint, timepoint_reg, expression_value, replicate)]
 
   # Add accession values as data attributes
   data.table::setattr(all_data, "ref", reference)
   data.table::setattr(all_data, "query", query)
 
-  # Add scaling as a data attribute
-  data.table::setattr(all_data, "scaling_method", scaling_method)
-
   # Results object
   results_list <- list(
     data = all_data,
-    model_comparison = model_comparison
+    model_comparison = model_comparison,
+    fun_args = fun_args
   )
 
-  return(results_list)
+  return(new_res_greatR(results_list))
 }
 
 #' Auxiliary function to apply registration with optimisation
@@ -285,12 +276,17 @@ register_manually <- function(data,
                               stretch,
                               shift,
                               loglik_separate,
+                              overlapping_percent = 0.5,
                               return_data_reg = TRUE) {
   # Apply registration
   data_reg <- apply_registration(data, stretch, shift)
 
   # Calculate model comparison
-  loglik_combined <- calc_loglik_H1(data_reg)
+  if (calc_overlapping_percent(data_reg) < overlapping_percent) {
+    loglik_combined <- -999
+  } else {
+    loglik_combined <- calc_loglik_H1(data_reg)
+  }
 
   # Model comparison
   model_comparison <- compare_H1_and_H2(data_reg, stretch, shift, loglik_combined, loglik_separate)
@@ -313,7 +309,7 @@ register_manually <- function(data,
 #' Explore manual search space for a single gene
 #'
 #' @noRd
-explore_manual_search_space <- function(data, stretches, shifts, loglik_separate) {
+explore_manual_search_space <- function(data, stretches, shifts, loglik_separate, overlapping_percent) {
   # Suppress "no visible binding for global variable" note
   BIC_diff <- NULL
 
@@ -324,7 +320,7 @@ explore_manual_search_space <- function(data, stretches, shifts, loglik_separate
       lapply(
         shifts,
         function(shift) {
-          results <- register_manually(data, stretch, shift, loglik_separate, return_data_reg = FALSE)
+          results <- register_manually(data, stretch, shift, loglik_separate, overlapping_percent, return_data_reg = FALSE)
           results$model_comparison
         }
       )
@@ -337,3 +333,13 @@ explore_manual_search_space <- function(data, stretches, shifts, loglik_separate
 
   return(best_params)
 }
+
+new_res_greatR <- function(x) {
+  structure(x, class = c("res_greatR", class(x)))
+}
+
+#' @export
+print.res_greatR <- function(x, ...) {
+  print(x$model_comparison)
+  return(invisible(x))
+}
