@@ -1,8 +1,8 @@
-brapa_sample_data <- data.table::fread(system.file("extdata/brapa_arabidopsis_all_replicates.csv", package = "greatR"))
+brapa_sample_data <- data.table::fread(system.file("extdata/brapa_arabidopsis_data.csv", package = "greatR"))
 reference <- "Ro18"
 query <- "Col0"
 gene_data <- brapa_sample_data[gene_id == "BRAA03G051930.3C"]
-all_data <- preprocess_data(gene_data, reference, query, scaling_method = "z-score")
+all_data <- suppressMessages(preprocess_data(gene_data, reference, query, scaling_method = "z-score"))
 
 test_that("match_names works", {
   a <- LETTERS[1:3]
@@ -33,6 +33,9 @@ test_that("cross_join works", {
   expect_equal(dim(dt_cj)[1], nrow(dt_a) * nrow(dt_b))
   expect_equal(dim(dt_cj)[2], ncol(dt_a) + ncol(dt_b))
   expect_equal(colnames(dt_cj), c(colnames(dt_a), colnames(dt_b)))
+  expect_equal(dt_cj$x, rep(dt_a$x, each = 3))
+  expect_equal(dt_cj$y, rep(dt_a$y, each = 3))
+  expect_equal(dt_cj$z, rep(dt_b$z, times = 2))
 })
 
 test_that("get_approximate_stretch works", {
@@ -56,9 +59,9 @@ test_that("get_search_space_limits (auto) works", {
   expect_equal(space_lims$stretch_init, 2.667, tolerance = 1e-2)
   expect_equal(space_lims$stretch_lower, 1.333, tolerance = 1e-2)
   expect_equal(space_lims$stretch_upper, 4, tolerance = 1e-2)
-  expect_equal(space_lims$shift_init, 0, tolerance = 1e-2)
-  expect_equal(space_lims$shift_lower, -20, tolerance = 1e-2)
-  expect_equal(space_lims$shift_upper, 16, tolerance = 1e-2)
+  expect_equal(space_lims$shift_init, -7.667, tolerance = 1e-2)
+  expect_equal(space_lims$shift_lower, -41, tolerance = 1e-2)
+  expect_equal(space_lims$shift_upper, 49.667, tolerance = 1e-2)
   expect_no_error(get_search_space_limits(all_data))
   expect_error(get_search_space_limits(gene_data))
 })
@@ -86,15 +89,15 @@ test_that("get_search_space_limits (init) works", {
   # Expected outputs
   expect_equal(names(space_lims), c("stretch_init", "stretch_lower", "stretch_upper", "shift_init", "shift_lower", "shift_upper"))
   expect_equal(space_lims$stretch_init, stretches, tolerance = 1e-2)
-  expect_equal(space_lims$stretch_lower, 1.333, tolerance = 1e-2)
-  expect_equal(space_lims$stretch_upper, 4, tolerance = 1e-2)
+  expect_equal(space_lims$stretch_lower, 0.5, tolerance = 1e-2)
+  expect_equal(space_lims$stretch_upper, 1.5, tolerance = 1e-2)
   expect_equal(space_lims$shift_init, shifts, tolerance = 1e-2)
-  expect_equal(space_lims$shift_lower, -20, tolerance = 1e-2)
-  expect_equal(space_lims$shift_upper, 16, tolerance = 1e-2)
+  expect_equal(space_lims$shift_lower, -1, tolerance = 1e-2)
+  expect_equal(space_lims$shift_upper, 33, tolerance = 1e-2)
 })
 
 test_that("calc_overlapping_percent works", {
-  all_data_reg <- apply_registration(all_data, 3.10, 2.13)
+  all_data_reg <- apply_registration(all_data, 3.10, -12.58)
   overlapping_raw <- calc_overlapping_percent(all_data)
   overlapping_reg <- calc_overlapping_percent(all_data_reg)
 
@@ -102,3 +105,55 @@ test_that("calc_overlapping_percent works", {
   expect_gte(overlapping_reg, overlapping_raw)
   expect_equal(overlapping_reg, 1, tolerance = 1e-1)
 })
+
+test_that("bind_results works", {
+  registration_results <- suppressMessages(register(
+    brapa_sample_data[gene_id %in% c("BRAA03G051930.3C", "BRAA04G005470.3C")],
+    reference = "Ro18",
+    query = "Col0",
+    scaling_method = "z-score",
+    stretches = 3.10,
+    shifts = -12.58,
+    use_optimisation = FALSE
+  ))
+
+  registration_results_a <- suppressMessages(register(
+    brapa_sample_data[gene_id == "BRAA03G051930.3C"],
+    reference = "Ro18",
+    query = "Col0",
+    scaling_method = "z-score",
+    stretches = 3.10,
+    shifts = -12.58,
+    use_optimisation = FALSE
+  ))
+
+  registration_results_b <- suppressMessages(register(
+    brapa_sample_data[gene_id == "BRAA04G005470.3C"],
+    reference = "Ro18",
+    query = "Col0",
+    scaling_method = "z-score",
+    stretches = 3.10,
+    shifts = -12.58,
+    use_optimisation = FALSE
+  ))
+
+  registration_results_c <- suppressMessages(register(
+    brapa_sample_data[gene_id == "BRAA04G005470.3C", ][, accession := ifelse(accession == "Ro18", "DH", "Col0")][],
+    reference = "DH",
+    query = "Col0",
+    scaling_method = "z-score",
+    stretches = 3.10,
+    shifts = -12.58,
+    use_optimisation = FALSE
+  ))
+
+  registration_results_ab1 <- bind_results(registration_results_a, registration_results_b)
+  registration_results_ab2 <- bind_results(list(registration_results_a, registration_results_b))
+
+  # Expected outputs
+  expect_s3_class(registration_results_ab1, "res_greatR")
+  expect_equal(registration_results_ab1$data, registration_results$data)
+  expect_equal(registration_results_ab1$model_comparison, registration_results$model_comparison)
+  expect_equal(registration_results_ab1, registration_results_ab2)
+  expect_error(bind_results(registration_results_a, registration_results_c))
+})
